@@ -32,6 +32,8 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
+#include <ctype.h>
 
 #define  XMLRPC_WANT_INTERNAL_DECLARATIONS
 #include "xmlrpc.h"
@@ -84,6 +86,121 @@
                          xml_element_children_size(elem)); \
     while (0)
 
+static xml_element *
+get_child_by_name (xmlrpc_env *env, xml_element *parent, char *name)
+{
+    size_t child_count, i;
+    xml_element **children;
+
+    children = xml_element_children(parent);
+    child_count = xml_element_children_size(parent);
+    for (i = 0; i < child_count; i++) {
+	if (0 == strcmp(xml_element_name(children[i]), name))
+	    return children[i];
+    }
+	
+    xmlrpc_env_set_fault_formatted(env, XMLRPC_PARSE_ERROR,
+				   "Expected <%s> to have child <%s>",
+				   xml_element_name(parent), name);
+    return NULL;
+}
+
+
+/*=========================================================================
+**  Number-Parsing Functions
+**=========================================================================
+**  These functions mirror atoi, atof, etc., but provide better
+**  error-handling.  These routines may reset errno to zero.
+*/
+
+static xmlrpc_int32
+xmlrpc_atoi (xmlrpc_env *env, char *str, size_t strlen,
+	     xmlrpc_int32 min, xmlrpc_int32 max)
+{
+    long i;
+    char *end;
+
+    XMLRPC_ASSERT_ENV_OK(env);
+    XMLRPC_ASSERT_PTR_OK(str);
+
+    /* Suppress compiler warnings. */
+    i = 0;
+
+    /* Check for leading white space. */
+    if (isspace(str[0]))
+	XMLRPC_FAIL1(env, XMLRPC_PARSE_ERROR,
+		     "\"%s\" must not contain whitespace", str);
+
+    /* Convert the value. */
+    end = str + strlen;
+    errno = 0;
+    i = strtol(str, &end, 10);
+
+    /* Look for ERANGE. */
+    if (errno != 0)
+	/* XXX - Do all operating systems have thread-safe strerror? */
+	XMLRPC_FAIL3(env, XMLRPC_PARSE_ERROR,
+		     "error parsing \"%s\": %s (%d)",
+		     str, strerror(errno), errno);
+
+    /* Look for out-of-range errors which didn't produce ERANGE. */
+    if (i < min || i > max)
+	XMLRPC_FAIL3(env, XMLRPC_PARSE_ERROR,
+		     "\"%s\" must be in range %d to %d", str, min, max);
+
+    /* Check for unused characters. */
+    if (end != str + strlen)
+	XMLRPC_FAIL1(env, XMLRPC_PARSE_ERROR,
+		     "\"%s\" contained trailing data", str);
+
+ cleanup:
+    errno = 0;
+    if (env->fault_occurred)
+	return 0;
+    return (xmlrpc_int32) i;
+}
+
+static double
+xmlrpc_atod (xmlrpc_env *env, char *str, size_t strlen)
+{
+    double d;
+    char *end;
+
+    XMLRPC_ASSERT_ENV_OK(env);
+    XMLRPC_ASSERT_PTR_OK(str);
+
+    /* Suppress compiler warnings. */
+    d = 0.0;
+
+    /* Check for leading white space. */
+    if (isspace(str[0]))
+	XMLRPC_FAIL1(env, XMLRPC_PARSE_ERROR,
+		     "\"%s\" must not contain whitespace", str);
+
+    /* Convert the value. */
+    end = str + strlen;
+    errno = 0;
+    d = strtod(str, &end);
+
+    /* Look for ERANGE. */
+    if (errno != 0)
+	/* XXX - Do all operating systems have thread-safe strerror? */
+	XMLRPC_FAIL3(env, XMLRPC_PARSE_ERROR,
+		     "error parsing \"%s\": %s (%d)",
+		     str, strerror(errno), errno);
+
+    /* Check for unused characters. */
+    if (end != str + strlen)
+	XMLRPC_FAIL1(env, XMLRPC_PARSE_ERROR,
+		     "\"%s\" contained trailing data", str);
+
+ cleanup:
+    errno = 0;
+    if (env->fault_occurred)
+	return 0.0;
+    return d;
+}
+
 
 /*=========================================================================
 **  make_string
@@ -94,7 +211,9 @@
 ** penalty, but ensures that we will never pass maliciously malformed
 ** UTF-8 data back up to the user layer, where it could wreak untold
 ** damange. Don't comment out this check unless you know *exactly* what
-** you're doing.
+** you're doing.  (Win32 developers who remove this check are *begging*
+** to wind up on BugTraq, because many of the Win32 filesystem routines
+** rely on an insecure UTF-8 decoder.)
 **
 ** XXX - This validation is redundant if the user chooses to convert
 ** UTF-8 data into a wchar_t string.
@@ -103,7 +222,10 @@
 static xmlrpc_value *
 make_string(xmlrpc_env *env, char *cdata, size_t cdata_size)
 {
+#ifdef HAVE_UNICODE_WCHAR
     xmlrpc_validate_utf8(env, cdata, cdata_size);
+#endif
+
     if (env->fault_occurred)
 	return NULL;
     return xmlrpc_build_value(env, "s#", cdata, cdata_size);
@@ -131,6 +253,8 @@ convert_value (xmlrpc_env *env, unsigned *depth, xml_element *elem)
     xmlrpc_mem_block *decoded;
     unsigned char *ascii_data;
     xmlrpc_value *retval;
+    xmlrpc_int32 i;
+    double d;
 
     XMLRPC_ASSERT_ENV_OK(env);
     XMLRPC_ASSERT(elem != NULL);
@@ -174,20 +298,25 @@ convert_value (xmlrpc_env *env, unsigned *depth, xml_element *elem)
 	    if (strcmp(child_name, "i4") == 0 ||
 		strcmp(child_name, "int") == 0)
 	    {
-		return xmlrpc_build_value(env, "i",
-					  (xmlrpc_int32) atoi(cdata));
+		i = xmlrpc_atoi(env, cdata, strlen(cdata),
+				XMLRPC_INT32_MIN, XMLRPC_INT32_MAX);
+		XMLRPC_FAIL_IF_FAULT(env);
+		return xmlrpc_build_value(env, "i", i);
 	    } else if (strcmp(child_name, "string") == 0) {
 		return make_string(env, cdata, cdata_size);
 	    } else if (strcmp(child_name, "boolean") == 0) {
-		return xmlrpc_build_value(env, "b", (xmlrpc_bool) atoi(cdata));
+		i = xmlrpc_atoi(env, cdata, strlen(cdata), 0, 1);
+		XMLRPC_FAIL_IF_FAULT(env);
+		return xmlrpc_build_value(env, "b", (xmlrpc_bool) i);
 	    } else if (strcmp(child_name, "double") == 0) {
-		return xmlrpc_build_value(env, "d", atof(cdata));
+		d = xmlrpc_atod(env, cdata, strlen(cdata));
+		XMLRPC_FAIL_IF_FAULT(env);
+		return xmlrpc_build_value(env, "d", d);
 	    } else if (strcmp(child_name, "dateTime.iso8601") == 0) {
 		return xmlrpc_build_value(env, "8", cdata);
 	    } else if (strcmp(child_name, "base64") == 0) {
 		/* No more tail calls once we do this! */
-		decoded = xmlrpc_base64_decode(env, (unsigned char*) cdata,
-					       cdata_size);
+		decoded = xmlrpc_base64_decode(env, cdata, cdata_size);
 		XMLRPC_FAIL_IF_FAULT(env);
 		ascii_data = XMLRPC_TYPED_MEM_BLOCK_CONTENTS(unsigned char,
 							     decoded);
@@ -306,8 +435,8 @@ convert_struct (xmlrpc_env *env, unsigned *depth, xml_element *elem)
 	CHECK_CHILD_COUNT(env, member, 2);
 
 	/* Get our key. */
-	name_elem = xml_element_children(member)[0];
-	CHECK_NAME(env, name_elem, "name");
+	name_elem = get_child_by_name(env, member, "name");
+	XMLRPC_FAIL_IF_FAULT(env);
 	CHECK_CHILD_COUNT(env, name_elem, 0);
 	cdata = xml_element_cdata(name_elem);
 	cdata_size = xml_element_cdata_size(name_elem);
@@ -315,7 +444,8 @@ convert_struct (xmlrpc_env *env, unsigned *depth, xml_element *elem)
 	XMLRPC_FAIL_IF_FAULT(env);
 
 	/* Get our value. */
-	value_elem = xml_element_children(member)[1];
+	value_elem = get_child_by_name(env, member, "value");
+	XMLRPC_FAIL_IF_FAULT(env);
 	value = convert_value(env, depth, value_elem);
 	XMLRPC_FAIL_IF_FAULT(env);
 
@@ -421,6 +551,7 @@ void xmlrpc_parse_call (xmlrpc_env *env,
     xml_element *call_elem, *name_elem, *params_elem;
     char *cdata;
     unsigned depth;
+    size_t call_child_count;
 
     XMLRPC_ASSERT_ENV_OK(env);
     XMLRPC_ASSERT(xml_data != NULL);
@@ -445,27 +576,40 @@ void xmlrpc_parse_call (xmlrpc_env *env,
 
     /* Pick apart and verify our structure. */
     CHECK_NAME(env, call_elem, "methodCall");
-    CHECK_CHILD_COUNT(env, call_elem, 2);
-    name_elem = xml_element_children(call_elem)[0];
-    params_elem = xml_element_children(call_elem)[1];
+    call_child_count = xml_element_children_size(call_elem);
+    if (call_child_count != 2 && call_child_count != 1)
+	XMLRPC_FAIL1(env, XMLRPC_PARSE_ERROR,
+		     "Expected <methodCall> to have 1 or 2 children, found %d",
+		     call_child_count);
 
     /* Extract the method name.
     ** SECURITY: We make sure the method name is valid UTF-8. */
-    CHECK_NAME(env, name_elem, "methodName");
+    name_elem = get_child_by_name(env, call_elem, "methodName");
+    XMLRPC_FAIL_IF_FAULT(env);
     CHECK_CHILD_COUNT(env, name_elem, 0);
     cdata = xml_element_cdata(name_elem);
+#ifdef HAVE_UNICODE_WCHAR
     xmlrpc_validate_utf8(env, cdata, strlen(cdata));
     XMLRPC_FAIL_IF_FAULT(env);
+#endif /* HAVE_UNICODE_WCHAR */
     *out_method_name = (char*) malloc(strlen(cdata) + 1);
     XMLRPC_FAIL_IF_NULL(*out_method_name, env, XMLRPC_INTERNAL_ERROR,
 			"Could not allocate memory for method name");
     strcpy(*out_method_name, cdata);
     
     /* Convert our parameters. */
-    depth = 0;
-    *out_param_array = convert_params(env, &depth, params_elem);
-    XMLRPC_ASSERT(depth == 0);
-    XMLRPC_FAIL_IF_FAULT(env);
+    if (call_child_count == 1) {
+	/* Workaround for Ruby XML-RPC and old versions of xmlrpc-epi. */
+	*out_param_array = xmlrpc_build_value(env, "()");
+	XMLRPC_FAIL_IF_FAULT(env);
+    } else {
+	params_elem = get_child_by_name(env, call_elem, "params");
+	XMLRPC_FAIL_IF_FAULT(env);
+	depth = 0;
+	*out_param_array = convert_params(env, &depth, params_elem);
+	XMLRPC_ASSERT(depth == 0);
+	XMLRPC_FAIL_IF_FAULT(env);
+    }
 
  cleanup:
     if (call_elem)
